==> lebiniou_3.32.orig.tar.gz: lebiniou-3.32/plugins/stable/main/delay_pp/delay_pp.c <==
/*
 *  Copyright 1994-2019 Olivier Girondel
 *
 *  This file is part of lebiniou.
 *
 *  lebiniou is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  lebiniou is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with lebiniou. If not, see <http://www.gnu.org/licenses/>.
 */

#include "context.h"
#include "particles.h"


u_long id = 1547215841;
u_long options = BE_SFX3D|BEQ_PARTICLES;
u_long mode = OVERLAY;
char desc[] = "Phase-space reconstruction using particles";


static const Point3d_t ORIGIN = { { 0.0, 0.0, 0.0 } };
static Particle_System_t *ps = NULL;


static void
Delay1_particles(Context_t *ctx)
{
  Buffer8_t *dst = passive_buffer(ctx);
  const Params3d_t *params3d = &ctx->params3d;
  Input_t *input = ctx->input;
  const uint8_t delay = Context_get_phase_space_delay(ctx);
  uint32_t x = 0;
  uint32_t y = delay;
  uint32_t z = 2 * delay;

  Buffer8_clear(dst);

  // move particles
  Particle_System_go(ps);

  pthread_mutex_lock(&input->mutex);
  for ( ; (z < input->size) && Particle_System_can_add(ps); x++, y++, z++) {
    Point3d_t p;
    p.pos.x = input->data[A_MONO][x];
    p.pos.y = input->data[A_MONO][y];
    p.pos.z = input->data[A_MONO][z];
    const float ttl = Input_random_float_range(input, 0.2, 1);
    Pixel_t color = Input_random_color(input);
    const Particle_t *part = Particle_new_indexed(ttl, color, p3d_mul(&p, 2.5), p3d_mul(&p, .1), ORIGIN, 0.0);

    Particle_System_add(ps, part);
  }
  pthread_mutex_unlock(&input->mutex);

  Particle_System_draw(ps, params3d, dst);
}


int8_t
create(Context_t *ctx)
{
  ps = Particle_System_new(PS_NOLIMIT);

  return 1;
}


void
destroy(Context_t *ctx)
{
  if (ps != NULL) {
    Particle_System_delete(ps);
  }
}


void
run(Context_t *ctx)
{
  Delay1_particles(ctx);
}

==> lebiniou_3.40.orig.tar.gz: lebiniou-3.40/plugins/stable/main/delay_pp/delay_pp.c <==
/*
 *  Copyright 1994-2020 Olivier Girondel
 *
 *  This file is part of lebiniou.
 *
 *  lebiniou is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  lebiniou is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with lebiniou. If not, see <http://www.gnu.org/licenses/>.
 */

#include "context.h"
#include "particles.h"
#include "parameters.h"


u_long id = 1547215841;
uint32_t version = 0;
u_long options = BE_SFX3D|BEQ_PARTICLES;
u_long mode = OVERLAY;
char desc[] = "Phase-space reconstruction using particles";


static const Point3d_t ORIGIN = { { 0.0, 0.0, 0.0 } };
static Particle_System_t *ps = NULL;

static float pos_factor = 0;
static float vel_factor = 0;
static float ttl_factor = 0;

json_t *
get_parameters()
{
  json_t *params = json_object();
  plugin_parameters_add_double(params, BPP_POS_FACTOR, pos_factor, -0.01, 0.01);
  plugin_parameters_add_double(params, BPP_VEL_FACTOR, vel_factor, -0.01, 0.01);
  plugin_parameters_add_double(params, BPP_TTL_FACTOR, ttl_factor, -0.01, 0.01);

  return params;
}


void
set_parameters(const json_t *in_parameters)
{
  plugin_parameter_parse_float_range(in_parameters, BPP_POS_FACTOR, &pos_factor, 0, 100);
  plugin_parameter_parse_float_range(in_parameters, BPP_VEL_FACTOR, &vel_factor, 0, 100);
  plugin_parameter_parse_float_range(in_parameters, BPP_TTL_FACTOR, &ttl_factor, 0, 100);
}


json_t *
parameters(const json_t *in_parameters)
{
  set_parameters(in_parameters);

  return get_parameters();
}


static void
Delay1_particles(Context_t *ctx)
{
  Buffer8_t *dst = passive_buffer(ctx);
  const Params3d_t *params3d = &ctx->params3d;
  Input_t *input = ctx->input;
  const uint8_t delay = Context_get_phase_space_delay(ctx);
  uint32_t x = 0;
  uint32_t y = delay;
  uint32_t z = 2 * delay;

  Buffer8_clear(dst);

  // move particles
  Particle_System_go(ps);

  pthread_mutex_lock(&input->mutex);
  for ( ; (z < input->size) && Particle_System_can_add(ps); x++, y++, z++) {
    Point3d_t p;
    p.pos.x = input->data[A_MONO][x];
    p.pos.y = input->data[A_MONO][y];
    p.pos.z = input->data[A_MONO][z];
    const float ttl = Input_random_float_range(input, 0.2, 1) * ttl_factor;
    Pixel_t color = Input_random_color(input);
    const Particle_t *part = Particle_new_indexed(ttl, color, p3d_mul(&p, pos_factor), p3d_mul(&p, vel_factor), ORIGIN, 0.0);

    Particle_System_add(ps, part);
  }
  pthread_mutex_unlock(&input->mutex);

  Particle_System_draw(ps, params3d, dst);
}


int8_t
create(Context_t *ctx)
{
  ps = Particle_System_new(PS_NOLIMIT);
  return 1;
}


void
destroy(Context_t *ctx)
{
  if (ps != NULL) {
    Particle_System_delete(ps);
  }
}


void
on_switch_on(Context_t *ctx)
{
  /* Initialize parameters */
  pos_factor = 2.5;
  vel_factor = 0.1;
  ttl_factor = 1.0;
}


void
run(Context_t *ctx)
{
  Delay1_particles(ctx);
}
